// Riesenie sustavy diferencialnych rovnic metodou Runge-Kutta
// Trieda SystemOfDifferentialEquations definuje vseobecne riesenie
// sustavy rovnic, konkretne funkcie f() a g() sustavy su definovane 
// v zdedenej podtriede. 

#include <math.h>
#include <iostream.hpp>

class SystemOfDifferentialEquations
{
  double x0;	// pociatocna podmienka pre x
  double y0;	// pociatocna podmienka pre y
  double t0;	// startovacia hodnota t
  double tn;	// konecna hodnota t

  double h;	// krok riesenia

  double xk;	// aktualne hodnoty 
  double yk;
  double tk;

  public:

  SystemOfDifferentialEquations(double x, double y, double ts, double te, double step);

  virtual double f( double t, double x, double y) = 0;
  virtual double g( double t, double x, double y) = 0;

  void init();	// nastavi pociatocne hodnoty 
  int step();  	// krok riesenia, vrati 0, ak je riesenie na konci

  double x();	// pristupove metody k premennym 
  double y();
  double t();
  
};

SystemOfDifferentialEquations::SystemOfDifferentialEquations
					(double x,double y,double ts,double te,double step)
{
  x0=x;
  y0=y;
  t0=ts;
  tn=te;
  h =step;

  tk=t0;	// init
  xk=x0;
  yk=y0;
}

void SystemOfDifferentialEquations::init()
{
  tk=t0;	// init
  xk=x0;
  yk=y0;
}


int SystemOfDifferentialEquations::step()
{

  if(tk > tn) return 0;
  

  double f1=f(tk,xk,yk);
  double g1=g(tk,xk,yk);

  double f2=f(tk+h*0.5, xk+h*0.5*f1, yk+h*0.5*g1);
  double g2=g(tk+h*0.5, xk+h*0.5*f1, yk+h*0.5*g1);

  double f3=f(tk+h*0.5, xk+h*0.5*f2, yk+h*0.5*g2);
  double g3=g(tk+h*0.5, xk+h*0.5*f2, yk+h*0.5*g2);

  double f4=f(tk+h,     xk+h*f3,     yk+h*g3);
  double g4=g(tk+h,     xk+h*f3,     yk+h*g3);

  xk=xk+h*(f1+2*f2+2*f3+f4)/6.0;
  yk=yk+h*(g1+2*g2+2*g3+g4)/6.0;

  tk=tk+h;
  
  return 1;
}

inline double SystemOfDifferentialEquations::x() { return xk; }
inline double SystemOfDifferentialEquations::y() { return yk; }
inline double SystemOfDifferentialEquations::t() { return tk; }

//
// Specializovana trieda - uzivatelom zadany problem
//

class MySystem : public SystemOfDifferentialEquations
{
  public:
  MySystem(double x,double y,double ts,double te,double step);

  double f( double t, double x, double y);
  double g( double t, double x, double y);
};

MySystem::MySystem(double x,double y,double ts,double te,double step)
        :SystemOfDifferentialEquations(x,y,ts,te,step)
{
}


double MySystem::f( double t, double x, double y)
{
  return y;
}

double MySystem::g( double t, double x, double y)
{
  return -1*x-1*y;
}


void main()
{
  //            x   vx
  //            x   y    ts   te   h
  MySystem ms(1.0, -5.0, 0.0, 5.0, 0.1);
  
  do{
    cout<<ms.t()<<"  "<<ms.x()<<"  "<<ms.y()<<endl;
  }  while( ms.step() );

}

